package GestureRecognition;

import be.ac.ulg.montefiore.run.jahmm.ObservationVector;
import be.ac.ulg.montefiore.run.jahmm.io.ObservationSequencesReader;
import be.ac.ulg.montefiore.run.jahmm.io.ObservationVectorReader;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.Reader;
import java.util.List;
import java.util.Vector;

/**
 * Created by IntelliJ IDEA.
 * User: MAD
 * Date: May 19, 2010
 * Time: 2:26:45 PM
 * PicFloyd
 */
public class ClassifierManager {
    private static ClassifierManager instance = null;
    private String configFile = "src/config.txt";
    private Vector<String> filesModel;
    private Vector<String> gestureName;
    private Classifier classifier;

    private ClassifierManager() {
        filesModel = new Vector<String>();
        classifier = new Classifier();
        gestureName = new Vector<String>();
        
        readConfiguration();
        constructClassifier();
    }
    
    public static ClassifierManager getInstance() {
        if (instance == null)
            instance = new ClassifierManager();
        return instance;
    }

    public void readConfiguration() {
        try {
            Reader reader = new FileReader(configFile);
            BufferedReader br = new BufferedReader(reader);

            String s;
            while ((s = br.readLine()) != null) {
                String[] tmp = s.split(":");
                if (tmp.length==2)
                {
                    gestureName.add(tmp[0].trim());
                    filesModel.add(tmp[1].trim());
                }
            }

        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    private void constructClassifier() {
        for (int i=0;i<filesModel.size();i++)
        {
            GestureModel model = new GestureModel();
            model.Learn(filesModel.get(i));
            classifier.addGestureModel(model);
        }
    }
    public String classify(List<ObservationVector> observation){
        Gesture gestureTest = new Gesture(observation);
        int gestRecognized = classifier.classifyGesture(gestureTest);
        if (gestRecognized<0)
            return "Gesture not recognized";
        else
            return gestureName.get(gestRecognized);
    }

    public void hmmTest(String fileName, int classNu) {
        try {
            System.out.println("======================For class "+classNu+" =====================");
            double recognized = 0;
            Reader reader = new FileReader(fileName);
            List<List<ObservationVector>> seqsTesting = null;
            seqsTesting = ObservationSequencesReader.readSequences(new ObservationVectorReader(), reader);
            if (seqsTesting.size() > 0) {
                for (int i = 0; i < seqsTesting.size(); i++) {
                    List<ObservationVector> test = seqsTesting.get(i);
                    Gesture gestureTest = new Gesture(test);
                    int gestRecognized = classifier.classifyGesture(gestureTest);
                    if (gestRecognized == classNu)
                        recognized++;
                    else
                        System.out.println("Recognized gesture: " + gestRecognized+" in fact was "+classNu);
                }
                double percentage0 = (double) recognized / (double) (seqsTesting.size());
                System.out.println("Percentage: " + percentage0);
            } else
                System.out.println("Input sequence null");
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}
