package GestureRecognition;

import be.ac.ulg.montefiore.run.jahmm.ObservationVector;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;

/**
 * User: mad - mad_bn@yahoo.com
 * Date: May 6, 2010
 * Time: 10:38:12 AM
 * PicFloyd
 */
public class Gesture {
    private boolean minmaxmanual = false;
    private double minacc;
    private double maxacc;

    private Vector<GestureEvent> data;

    public Gesture() {
        this.data = new Vector<GestureEvent>();
    }

    public Gesture(List<ObservationVector> gesture) {
        data = new Vector<GestureEvent>();
        Iterator iter = gesture.iterator();
        while (iter.hasNext()) {
            ObservationVector obs = (ObservationVector) iter.next();
            GestureEvent ge = new GestureEvent(obs);
            data.add(ge);
        }
    }

    /**
     * Make a deep copy of another Gesture object.
     *
     * @param original Another Gesture object
     */
    public Gesture(Gesture original) {
        this.data = new Vector<GestureEvent>();
        Vector<GestureEvent> origin = original.getData();
        for (int i = 0; i < origin.size(); i++) {
            this.add((GestureEvent) origin.get(i));
        }
    }


    public void add(GestureEvent event) {
        this.data.add(event);
    }

    public GestureEvent getLastData() {
        return (GestureEvent) this.data.get(this.data.size() - 1);
    }

    public Vector<GestureEvent> getData() {
        return this.data;
    }

    /**
     * Removes the first element of the acceleration queue of a gesture
     */
    public void removeFirstData() {
        this.data.remove(0);
    }

    public int getCountOfData() {
        return this.data.size();
    }

    public void setMaxAndMinAcceleration(double max, double min) {
        this.maxacc = max;
        this.minacc = min;
        this.minmaxmanual = true;
    }

    public double getMaxAcceleration() {
        if (!this.minmaxmanual) {
            double maxacc = Double.MIN_VALUE;
            for (int i = 0; i < this.data.size(); i++) {
                if (Math.abs(this.data.get(i).getXAcc()) > maxacc) {
                    maxacc = Math.abs(this.data.get(i).getXAcc());
                }
                if (Math.abs(this.data.get(i).getYAcc()) > maxacc) {
                    maxacc = Math.abs(this.data.get(i).getYAcc());
                }
                if (Math.abs(this.data.get(i).getZAcc()) > maxacc) {
                    maxacc = Math.abs(this.data.get(i).getZAcc());
                }
            }
            return maxacc;
        } else {
            return this.maxacc;
        }
    }

    public double getMinAcceleration() {
        if (!this.minmaxmanual) {
            double minacc = Double.MAX_VALUE;
            for (int i = 0; i < this.data.size(); i++) {
                if (Math.abs(this.data.get(i).getXAcc()) < minacc) {
                    minacc = Math.abs(this.data.get(i).getXAcc());
                }
                if (Math.abs(this.data.get(i).getYAcc()) < minacc) {
                    minacc = Math.abs(this.data.get(i).getYAcc());
                }
                if (Math.abs(this.data.get(i).getZAcc()) < minacc) {
                    minacc = Math.abs(this.data.get(i).getZAcc());
                }
            }
            return minacc;
        } else {
            return this.minacc;
        }
    }

    public List<ObservationVector> toList() {
        List<ObservationVector> gestureList = new Vector<ObservationVector>();
        Iterator iter = data.iterator();

        while (iter.hasNext()) {
            gestureList.add(((GestureEvent) iter.next()).getObservationVector());
        }
        return gestureList;
    }
}
