import GestureRecognition.Classifier;
import GestureRecognition.Gesture;
import GestureRecognition.GestureModel;
import IO.IOHMMOperations;
import be.ac.ulg.montefiore.run.jahmm.Hmm;
import be.ac.ulg.montefiore.run.jahmm.ObservationVector;
import be.ac.ulg.montefiore.run.jahmm.OpdfMultiGaussian;
import be.ac.ulg.montefiore.run.jahmm.OpdfMultiGaussianFactory;
import be.ac.ulg.montefiore.run.jahmm.io.HmmWriter;
import be.ac.ulg.montefiore.run.jahmm.io.ObservationSequencesReader;
import be.ac.ulg.montefiore.run.jahmm.io.ObservationVectorReader;
import be.ac.ulg.montefiore.run.jahmm.io.OpdfWriter;
import be.ac.ulg.montefiore.run.jahmm.learn.BaumWelchLearner;

import java.io.FileReader;
import java.io.FileWriter;
import java.io.Reader;
import java.io.Writer;
import java.util.List;

/**
 * User: mad - mad_bn@yahoo.com
 * Date: May 5, 2010
 * Time: 2:23:56 PM
 * PICFLOYD
 */
public class Test {

    public static void main(String args[]) {
        Classifier classifier = new Classifier();

        System.out.println("Learning model 1");
        GestureModel model1 = new GestureModel();

        model1.Learn("src/TestFiles/circleTraining.txt");

        System.out.println("Learning model 2");
        GestureModel model2 = new GestureModel();
        model2.Learn("src/TestFiles/circleTraining.txt");


        System.out.println("Learning model 3");
        GestureModel model3 = new GestureModel();
        model3.Learn("src/TestFiles/spiralTraining.txt");

        System.out.println("Learning model 4");
        GestureModel model4 = new GestureModel();
        model4.Learn("src/TestFiles/sLetterTraining.txt");


        classifier.addGestureModel(model1);
        classifier.addGestureModel(model2);
        classifier.addGestureModel(model3);
        classifier.addGestureModel(model4);

        System.out.println("Testing");
        hmmTest(classifier, "src/TestFiles/circleTesting.txt", 0);
        hmmTest(classifier, "src/TestFiles/squareTesting.txt", 1);
        hmmTest(classifier, "src/TestFiles/spiralTesting.txt", 2);
        hmmTest(classifier, "src/TestFiles/sLetterTesting.txt", 3);

    }

    public static void hmmTest(Classifier classifier, String fileName, int classNu) {
        try {
            System.out.println("======================For class "+classNu+" =====================");
            double recognized = 0;
            Reader reader = new FileReader(fileName);
            List<List<ObservationVector>> seqsTesting = null;
            seqsTesting = ObservationSequencesReader.readSequences(new ObservationVectorReader(), reader);
            if (seqsTesting.size() > 0) {
                for (int i = 0; i < seqsTesting.size(); i++) {
                    List<ObservationVector> test = seqsTesting.get(i);
                    Gesture gestureTest = new Gesture(test);
                    int gestRecognized = classifier.classifyGesture(gestureTest);
                    if (gestRecognized == classNu)
                        recognized++;
                    else
                        System.out.println("Recognized gesture: " + gestRecognized+" in fact was "+classNu);
                }
                double percentage0 = (double) recognized / (double) (seqsTesting.size());
                System.out.println("Percentage: " + percentage0);
            } else
                System.out.println("Input sequence null");
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}
